@@ -14,43 +14,116 @@
 #include "kxmlguiwindow.h"
 #include "debug.h"
 
-#include "kmainwindow_p.h"
-#include "kmessagebox.h"
 #include "kactioncollection.h"
+#include "kmainwindow_p.h"
+#include <KMessageBox>
 #ifdef QT_DBUS_LIB
 #include "kmainwindowiface_p.h"
 #endif
-#include "ktoolbarhandler_p.h"
-#include "kxmlguifactory.h"
 #include "kedittoolbar.h"
 #include "khelpmenu.h"
 #include "ktoolbar.h"
+#include "ktoolbarhandler_p.h"
+#include "kxmlguifactory.h"
 
 #ifdef QT_DBUS_LIB
 #include <QDBusConnection>
 #endif
 #include <QDomDocument>
+#include <QEvent>
+#include <QList>
 #include <QMenuBar>
 #include <QStatusBar>
 #include <QWidget>
-#include <QList>
-#include <QEvent>
 
-#include <ktoggleaction.h>
-#include <kstandardaction.h>
-#include <kconfig.h>
-#include <klocalizedstring.h>
-#include <kaboutdata.h>
-#include <ksharedconfig.h>
-#include <kconfiggroup.h>
+#include <KAboutData>
+#include <KCommandBar>
+#include <KConfig>
+#include <KConfigGroup>
+#include <KLocalizedString>
+#include <KSharedConfig>
+#include <KStandardAction>
+#include <KToggleAction>
 
-#include <stdlib.h>
-#include <ctype.h>
+#include <cctype>
+#include <cstdlib>
+
+/**
+ * A helper function that takes a list of KActionCollection* and converts it
+ * to KCommandBar::ActionGroup
+ */
+static QVector<KCommandBar::ActionGroup> actionCollectionToActionGroup(const std::vector<KActionCollection *> &actionCollections)
+{
+    using ActionGroup = KCommandBar::ActionGroup;
+
+    QVector<ActionGroup> actionList;
+    actionList.reserve(actionCollections.size());
+
+    for (const auto collection : actionCollections) {
+        const QList<QAction *> collectionActions = collection->actions();
+        const QString componentName = collection->componentDisplayName();
+
+        ActionGroup ag;
+        ag.name = componentName;
+        ag.actions.reserve(collection->count());
+        for (const auto action : collectionActions) {
+            /**
+             * If this action is a menu, fetch all its child actions
+             * and skip the menu action itself
+             */
+            if (QMenu *menu = action->menu()) {
+                const QList<QAction *> menuActions = menu->actions();
+
+                ActionGroup menuActionGroup;
+                menuActionGroup.name = KLocalizedString::removeAcceleratorMarker(action->text());
+                menuActionGroup.actions.reserve(menuActions.size());
+                for (const auto mAct : menuActions) {
+                    if (mAct) {
+                        menuActionGroup.actions.append(mAct);
+                    }
+                }
+
+                /**
+                 * If there were no actions in the menu, we
+                 * add the menu to the list instead because it could
+                 * be that the actions are created on demand i.e., aboutToShow()
+                 */
+                if (!menuActions.isEmpty()) {
+                    actionList.append(menuActionGroup);
+                    continue;
+                }
+            }
+
+            if (action && !action->text().isEmpty()) {
+                ag.actions.append(action);
+            }
+        }
+        actionList.append(ag);
+    }
+    return actionList;
+}
+
+static void getActionCollections(KXMLGUIClient *client, std::vector<KActionCollection *> &actionCollections)
+{
+    if (!client) {
+        return;
+    }
+
+    auto actionCollection = client->actionCollection();
+    if (actionCollection && !actionCollection->isEmpty()) {
+        actionCollections.push_back(client->actionCollection());
+    }
+
+    const QList<KXMLGUIClient *> childClients = client->childClients();
+    for (auto child : childClients) {
+        getActionCollections(child, actionCollections);
+    }
+}
 
 class KXmlGuiWindowPrivate : public KMainWindowPrivate
 {
 public:
-    void _k_slotFactoryMakingChanges(bool b)
+    void slotFactoryMakingChanges(bool b)
     {
         // While the GUI factory is adding/removing clients,
         // don't let KMainWindow think those are changes made by the user
@@ -58,7 +131,11 @@ public:
         letDirtySettings = !b;
     }
 
-    bool showHelpMenu: 1;
+    bool commandBarEnabled = true;
+    // Last executed actions in command bar
+    QVector<QString> lastExecutedActions;
+
+    bool showHelpMenu : 1;
     QSize defaultSize;
 
     KDEPrivate::ToolBarHandler *toolBarHandler;
@@ -68,9 +145,10 @@ public:
 };
 
 KXmlGuiWindow::KXmlGuiWindow(QWidget *parent, Qt::WindowFlags f)
-    : KMainWindow(*new KXmlGuiWindowPrivate, parent, f), KXMLGUIBuilder(this)
+    : KMainWindow(*new KXmlGuiWindowPrivate, parent, f)
+    , KXMLGUIBuilder(this)
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     d->showHelpMenu = true;
     d->toolBarHandler = nullptr;
     d->showStatusBarAction = nullptr;
@@ -78,11 +156,44 @@ KXmlGuiWindow::KXmlGuiWindow(QWidget *parent, Qt::WindowFlags f)
 #ifdef QT_DBUS_LIB
     new KMainWindowInterface(this);
 #endif
+
+    /**
+     * Set up KCommandBar launcher action
+     */
+    auto a = actionCollection()->addAction(QStringLiteral("open_kcommand_bar"), this, [this] {
+        /**
+         * Do nothing when command bar is disabled
+         */
+        if (!isCommandBarEnabled()) {
+            return;
+        }
+
+        auto ac = actionCollection();
+        if (!ac) {
+            return;
+        }
+
+        KCommandBar kc(this);
+        std::vector<KActionCollection *> actionCollections;
+        const auto clients = guiFactory()->clients();
+        actionCollections.reserve(clients.size());
+
+        // Grab action collections recursively
+        for (const auto &client : clients) {
+            getActionCollections(client, actionCollections);
+        }
+
+        kc.setActions(actionCollectionToActionGroup(actionCollections));
+        kc.exec();
+    });
+    a->setIcon(QIcon::fromTheme(QStringLiteral("search")));
+    a->setText(i18n("Find Action…"));
+    actionCollection()->setDefaultShortcut(a, QKeySequence(Qt::CTRL | Qt::ALT | Qt::Key_I));
 }
 
 QAction *KXmlGuiWindow::toolBarMenuAction()
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     if (!d->toolBarHandler) {
         return nullptr;
     }
@@ -92,7 +203,7 @@ QAction *KXmlGuiWindow::toolBarMenuAction()
 
 void KXmlGuiWindow::setupToolbarMenuActions()
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     if (d->toolBarHandler) {
         d->toolBarHandler->setupActions();
     }
@@ -100,7 +211,7 @@ void KXmlGuiWindow::setupToolbarMenuActions()
 
 KXmlGuiWindow::~KXmlGuiWindow()
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     delete d->factory;
 }
 
@@ -109,12 +220,14 @@ bool KXmlGuiWindow::event(QEvent *ev)
     bool ret = KMainWindow::event(ev);
     if (ev->type() == QEvent::Polish) {
 #ifdef QT_DBUS_LIB
-        QDBusConnection::sessionBus().registerObject(dbusName() + QLatin1String("/actions"), actionCollection(),
-                QDBusConnection::ExportScriptableSlots |
-                QDBusConnection::ExportScriptableProperties |
-                QDBusConnection::ExportNonScriptableSlots |
-                QDBusConnection::ExportNonScriptableProperties |
-                QDBusConnection::ExportChildObjects);
+        /* clang-format off */
+        constexpr auto opts = QDBusConnection::ExportScriptableSlots
+                              | QDBusConnection::ExportScriptableProperties
+                              | QDBusConnection::ExportNonScriptableSlots
+                              | QDBusConnection::ExportNonScriptableProperties
+                              | QDBusConnection::ExportChildObjects;
+        /* clang-format on */
+        QDBusConnection::sessionBus().registerObject(dbusName() + QLatin1String("/actions"), actionCollection(), opts);
 #endif
     }
     return ret;
@@ -122,37 +235,37 @@ bool KXmlGuiWindow::event(QEvent *ev)
 
 void KXmlGuiWindow::setHelpMenuEnabled(bool showHelpMenu)
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     d->showHelpMenu = showHelpMenu;
 }
 
 bool KXmlGuiWindow::isHelpMenuEnabled() const
 {
-    K_D(const KXmlGuiWindow);
+    Q_D(const KXmlGuiWindow);
     return d->showHelpMenu;
 }
 
 KXMLGUIFactory *KXmlGuiWindow::guiFactory()
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     if (!d->factory) {
         d->factory = new KXMLGUIFactory(this, this);
-        connect(d->factory, SIGNAL(makingChanges(bool)),
-                this, SLOT(_k_slotFactoryMakingChanges(bool)));
+        connect(d->factory, &KXMLGUIFactory::makingChanges, this, [d](bool state) {
+            d->slotFactoryMakingChanges(state);
+        });
     }
     return d->factory;
 }
 
 void KXmlGuiWindow::configureToolbars()
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     KConfigGroup cg(KSharedConfig::openConfig(), "");
     saveMainWindowSettings(cg);
     if (!d->toolBarEditor) {
         d->toolBarEditor = new KEditToolBar(guiFactory(), this);
         d->toolBarEditor->setAttribute(Qt::WA_DeleteOnClose);
-        connect(d->toolBarEditor, &KEditToolBar::newToolBarConfig,
-                this, &KXmlGuiWindow::saveNewToolbarConfig);
+        connect(d->toolBarEditor, &KEditToolBar::newToolBarConfig, this, &KXmlGuiWindow::saveNewToolbarConfig);
     }
     d->toolBarEditor->show();
 }
@@ -175,11 +288,10 @@ void KXmlGuiWindow::setupGUI(StandardWindowOptions options, const QString &xmlfi
 
 void KXmlGuiWindow::setupGUI(const QSize &defaultSize, StandardWindowOptions options, const QString &xmlfile)
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
 
     if (options & Keys) {
-        KStandardAction::keyBindings(guiFactory(),
-                                     SLOT(configureShortcuts()), actionCollection());
+        KStandardAction::keyBindings(guiFactory(), &KXMLGUIFactory::showConfigureShortcutsDialog, actionCollection());
     }
 
     if ((options & StatusBar) && statusBar()) {
@@ -188,8 +300,7 @@ void KXmlGuiWindow::setupGUI(const QSize &defaultSize, StandardWindowOptions opt
 
     if (options & ToolBar) {
         setStandardToolBarMenuEnabled(true);
-        KStandardAction::configureToolbars(this,
-                                           SLOT(configureToolbars()), actionCollection());
+        KStandardAction::configureToolbars(this, &KXmlGuiWindow::configureToolbars, actionCollection());
     }
 
     d->defaultSize = defaultSize;
@@ -215,9 +326,9 @@ void KXmlGuiWindow::setupGUI(const QSize &defaultSize, StandardWindowOptions opt
 }
 void KXmlGuiWindow::createGUI(const QString &xmlfile)
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     // disabling the updates prevents unnecessary redraws
-    //setUpdatesEnabled( false );
+    // setUpdatesEnabled( false );
 
     // just in case we are rebuilding, let's remove our old client
     guiFactory()->removeClient(this);
@@ -228,7 +339,7 @@ void KXmlGuiWindow::createGUI(const QString &xmlfile)
         mb->clear();
     }
 
-    qDeleteAll(toolBars());   // delete all toolbars
+    qDeleteAll(toolBars()); // delete all toolbars
 
     // don't build a help menu unless the user ask for it
     if (d->showHelpMenu) {
@@ -273,8 +384,8 @@ void KXmlGuiWindow::createGUI(const QString &xmlfile)
     // Help beginners who call setXMLFile and then setupGUI...
     if (!xmlFile().isEmpty() && xmlFile() != windowXmlFile) {
         qCWarning(DEBUG_KXMLGUI) << "You called setXMLFile(" << xmlFile() << ") and then createGUI or setupGUI,"
-                   << "which also calls setXMLFile and will overwrite the file you have previously set.\n"
-                   << "You should call createGUI(" << xmlFile() << ") or setupGUI(<options>," << xmlFile() << ") instead.";
+                                 << "which also calls setXMLFile and will overwrite the file you have previously set.\n"
+                                 << "You should call createGUI(" << xmlFile() << ") or setupGUI(<options>," << xmlFile() << ") instead.";
     }
 
     // we always want to load in our global standards file
@@ -300,16 +411,14 @@ void KXmlGuiWindow::slotStateChanged(const QString &newstate)
     stateChanged(newstate, KXMLGUIClient::StateNoReverse);
 }
 
-void KXmlGuiWindow::slotStateChanged(const QString &newstate,
-                                     bool reverse)
+void KXmlGuiWindow::slotStateChanged(const QString &newstate, bool reverse)
 {
-    stateChanged(newstate,
-                 reverse ? KXMLGUIClient::StateReverse : KXMLGUIClient::StateNoReverse);
+    stateChanged(newstate, reverse ? KXMLGUIClient::StateReverse : KXMLGUIClient::StateNoReverse);
 }
 
 void KXmlGuiWindow::setStandardToolBarMenuEnabled(bool enable)
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     if (enable) {
         if (d->toolBarHandler) {
             return;
@@ -336,18 +445,17 @@ void KXmlGuiWindow::setStandardToolBarMenuEnabled(bool enable)
 
 bool KXmlGuiWindow::isStandardToolBarMenuEnabled() const
 {
-    K_D(const KXmlGuiWindow);
+    Q_D(const KXmlGuiWindow);
     return (d->toolBarHandler);
 }
 
 void KXmlGuiWindow::createStandardStatusBarAction()
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     if (!d->showStatusBarAction) {
         d->showStatusBarAction = KStandardAction::showStatusbar(this, &KMainWindow::setSettingsDirty, actionCollection());
         QStatusBar *sb = statusBar(); // Creates statusbar if it doesn't exist already.
-        connect(d->showStatusBarAction, &QAction::toggled,
-                sb, &QWidget::setVisible);
+        connect(d->showStatusBarAction, &QAction::toggled, sb, &QWidget::setVisible);
         d->showStatusBarAction->setChecked(sb->isHidden());
     } else {
         // If the language has changed, we'll need to grab the new text and whatsThis
@@ -371,7 +479,7 @@ void KXmlGuiWindow::finalizeGUI(bool /*force*/)
 
 void KXmlGuiWindow::applyMainWindowSettings(const KConfigGroup &config)
 {
-    K_D(KXmlGuiWindow);
+    Q_D(KXmlGuiWindow);
     KMainWindow::applyMainWindowSettings(config);
     QStatusBar *sb = findChild<QStatusBar *>();
     if (sb && d->showStatusBarAction) {
@@ -379,7 +487,7 @@ void KXmlGuiWindow::applyMainWindowSettings(const KConfigGroup &config)
     }
 }
 
-// KDE5 TODO: change it to "using KXMLGUIBuilder::finalizeGUI;" in the header
+// TODO KF6: change it to "using KXMLGUIBuilder::finalizeGUI;" in the header
 // and remove the reimplementation
 void KXmlGuiWindow::finalizeGUI(KXMLGUIClient *client)
 {
@@ -388,7 +496,7 @@ void KXmlGuiWindow::finalizeGUI(KXMLGUIClient *client)
 
 void KXmlGuiWindow::checkAmbiguousShortcuts()
 {
-    QMap<QString, QAction*> shortcuts;
+    QMap<QString, QAction *> shortcuts;
     QAction *editCutAction = actionCollection()->action(QStringLiteral("edit_cut"));
     QAction *deleteFileAction = actionCollection()->action(QStringLiteral("deletefile"));
     const auto actions = actionCollection()->actions();
@@ -406,8 +514,8 @@ void KXmlGuiWindow::checkAmbiguousShortcuts()
                     // There is one exception, if the conflicting shortcut is a non primary shortcut of "edit_cut"
                     // and "deleteFileAction" is the other action since Shift+Delete is used for both in our default code
                     bool showWarning = true;
-                    if ((action == editCutAction && existingShortcutAction == deleteFileAction) ||
-                        (action == deleteFileAction && existingShortcutAction == editCutAction)) {
+                    if ((action == editCutAction && existingShortcutAction == deleteFileAction)
+                        || (action == deleteFileAction && existingShortcutAction == editCutAction)) {
                         QList<QKeySequence> editCutActionShortcuts = editCutAction->shortcuts();
                         if (editCutActionShortcuts.indexOf(shortcut) > 0) // alternate shortcut
                         {
@@ -423,7 +531,15 @@ void KXmlGuiWindow::checkAmbiguousShortcuts()
                         const QString existingShortcutActionName = KLocalizedString::removeAcceleratorMarker(existingShortcutAction->text());
                         QString dontShowAgainString = existingShortcutActionName + actionName + shortcut.toString();
                         dontShowAgainString.remove(QLatin1Char('\\'));
-                        KMessageBox::information(this, i18n("There are two actions (%1, %2) that want to use the same shortcut (%3). This is most probably a bug. Please report it in <a href='https://bugs.kde.org'>bugs.kde.org</a>", existingShortcutActionName, actionName, shortcut.toString(QKeySequence::NativeText)), i18n("Ambiguous Shortcuts"), dontShowAgainString, KMessageBox::Notify | KMessageBox::AllowLink);
+                        KMessageBox::information(this,
+                                                 i18n("There are two actions (%1, %2) that want to use the same shortcut (%3). This is most probably a bug. "
+                                                      "Please report it in <a href='https://bugs.kde.org'>bugs.kde.org</a>",
+                                                      existingShortcutActionName,
+                                                      actionName,
+                                                      shortcut.toString(QKeySequence::NativeText)),
+                                                 i18n("Ambiguous Shortcuts"),
+                                                 dontShowAgainString,
+                                                 KMessageBox::Notify | KMessageBox::AllowLink);
                     }
                 } else {
                     shortcuts.insert(portableShortcutText, action);
@@ -433,5 +549,26 @@ void KXmlGuiWindow::checkAmbiguousShortcuts()
     }
 }
 
-#include "moc_kxmlguiwindow.cpp"
+void KXmlGuiWindow::setCommandBarEnabled(bool enable)
+{
+    /**
+     * Unset the shortcut
+     */
+    auto cmdBarAction = actionCollection()->action(QStringLiteral("open_kcommand_bar"));
+    if (enable) {
+        actionCollection()->setDefaultShortcut(cmdBarAction, Qt::CTRL | Qt::ALT | Qt::Key_I);
+    } else {
+        actionCollection()->setDefaultShortcut(cmdBarAction, {});
+    }
 
+    Q_D(KXmlGuiWindow);
+    d->commandBarEnabled = enable;
+}
+
+bool KXmlGuiWindow::isCommandBarEnabled() const
+{
+    Q_D(const KXmlGuiWindow);
+    return d->commandBarEnabled;
+}
+
+#include "moc_kxmlguiwindow.cpp"
